from .locals import get_cid


def cid_context_processor(request):
    """
    Adds the correlation id as ``correlation_id`` to template contexts
    """
    cid = get_cid()
    if cid:
        return {'correlation_id': get_cid()}
    return {}



from threading import local
import uuid

from django.conf import settings


_thread_locals = local()


def set_cid(cid):
    """Set the correlation id for the current request."""
    setattr(_thread_locals, 'CID', cid)


def get_cid():
    """Return the currently set correlation id (if any).

    If no correlation id has been set and ``CID_GENERATE`` is enabled
    in the settings, a new correlation id is set and returned.
    """
    cid = getattr(_thread_locals, 'CID', None)
    if cid is None and getattr(settings, 'CID_GENERATE', False):
        cid = str(uuid.uuid4())
        set_cid(cid)
    return cid



from django.conf import settings

from cid.locals import get_cid
from cid.locals import set_cid


class CidMiddleware:
    """
    Middleware class to extract the correlation id from incoming headers
    and add them to outgoing headers
    """

    def __init__(self, get_response):
        self.get_response = get_response
        self.cid_request_header = getattr(
            settings, 'CID_HEADER', 'X_CORRELATION_ID'
        )
        self.cid_response_header = getattr(
            settings, 'CID_RESPONSE_HEADER', self.cid_request_header
        )

    def _process_request(self, request):
        cid = request.META.get(self.cid_request_header, None)
        if cid is None:
            cid = get_cid()
        request.correlation_id = cid
        set_cid(cid)
        return request

    def _process_response(self, response):
        cid = get_cid()
        if cid and self.cid_response_header:
            response[self.cid_response_header] = cid
        return response

    def __call__(self, request):
        request = self._process_request(request)
        response = self.get_response(request)
        return self._process_response(response)


# FIXME: to be removed once we remove support of Django 1.11
class CidOldStyleMiddleware(CidMiddleware):
    """Support for the old ``MIDDLEWARE_CLASSES`` setting."""

    def __init__(self):
        print('middleware init')
        # `get_response` is only used in `CidMiddleware.__call__`,
        # which is not called when using `MIDDLEWARE_CLASSES`.
        super().__init__(get_response='dummy')

    def process_request(self, request):
        self._process_request(request)
        # We must return None otherwise Django thinks it's the
        # response.
        return None

    def process_response(self, request, response):
        return self._process_response(response)



from django.db.backends.oracle.base import DatabaseWrapper as BaseDatabaseWrapper

from ...cursor import CidCursorWrapper


class DatabaseWrapper(BaseDatabaseWrapper):
    # Don't warn about abstract `_start_transaction_under_autocommit`
    # pylint: disable=abstract-method

    def create_cursor(self, name=None):
        base_cursor = super().create_cursor(name)
        return CidCursorWrapper(base_cursor)



from django.conf import settings

from .locals import get_cid


DEFAULT_CID_SQL_COMMENT_TEMPLATE = 'cid: {cid}'


class CidCursorWrapper:
    """
    A cursor wrapper that attempts to add a cid comment to each query
    """
    def __init__(self, cursor):
        self.cursor = cursor

    def __getattr__(self, attr):
        if attr in self.__dict__:
            return self.__dict__[attr]
        return getattr(self.cursor, attr)

    def __iter__(self):
        return iter(self.cursor)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def add_comment(self, sql):
        cid_sql_template = getattr(
            settings, 'CID_SQL_COMMENT_TEMPLATE', DEFAULT_CID_SQL_COMMENT_TEMPLATE
        )
        cid = get_cid()
        if not cid:
            return sql
        # FIXME (dbaty): we could use "--" prefixed comments so that
        # we would not have to bother with escaping the cid (assuming
        # it does not contain newline characters).
        cid = cid.replace('/*', r'\/\*').replace('*/', r'\*\/')
        return "/* {} */\n{}".format(cid_sql_template.format(cid=cid), sql)

    # The following methods cannot be implemented in __getattr__, because the
    # code must run when the method is invoked, not just when it is accessed.

    def callproc(self, procname, params=None):
        return self.cursor.callproc(procname, params)

    def execute(self, sql, params=None):
        sql = self.add_comment(sql)
        return self.cursor.execute(sql, params)

    def executemany(self, sql, param_list):
        sql = self.add_comment(sql)
        return self.cursor.executemany(sql, param_list)



from setuptools import setup, find_packages


readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='django-cid',
    version='1.0',
    description="""Correlation IDs in Django for debugging requests""",
    long_description=readme + '\n\n' + history,
    author='Snowball One',
    author_email='opensource+django-cid@polyconseil.fr',
    maintainer="Polyconseil",
    maintainer_email="opensource+django-cid@polyconseil.fr",
    url='https://github.com/Polyconseil/django-cid',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django>=1.11',
    ],
    license="BSD",
    zip_safe=False,
    keywords='django logging correlation id debugging',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
